from tira.io_utils import huggingface_model_mounts, _ln_huggingface_model_mounts
import unittest
import os
from pathlib import Path

class MountHfModelTest(unittest.TestCase):
    def test_none_returns_empty_dict(self):
        expected = {}
        actual = huggingface_model_mounts(None)
        self.assertEqual(actual, expected)

    def test_empty_list_returns_empty_dict(self):
        expected = {}
        actual = huggingface_model_mounts([])
        self.assertEqual(actual, expected)

    def test_exception_is_thrown_for_non_existing_model(self):
        with self.assertRaises(Exception) as context:
            huggingface_model_mounts(['model-does-not-exist'])
        self.assertTrue('model-does-not-exist' in str(context.exception))

    def test_mounts_for_single_existing_model(self):
        expected = {'models--openai-community--gpt2': {'bind': '/root/.cache/huggingface/hub/models--openai-community--gpt2', 'mode': 'ro'}}
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = huggingface_model_mounts(['openai-community/gpt2'])
        actual = {str(k).split('/')[-1]: v for k, v in actual.items()}
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)

    def test_mounts_for_multiple_existing_model(self):
        expected = {
            'models--openai-community--gpt2': {'bind': '/root/.cache/huggingface/hub/models--openai-community--gpt2', 'mode': 'ro'},
            'models--openai-community--gpt2-large': {'bind': '/root/.cache/huggingface/hub/models--openai-community--gpt2-large', 'mode': 'ro'},
            }
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = huggingface_model_mounts(['openai-community/gpt2', 'openai-community/gpt2-large'])
        actual = {str(k).split('/')[-1]: v for k, v in actual.items()}
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)

    def test_mounts_for_multiple_redundant_existing_model(self):
        expected = {
            'models--openai-community--gpt2': {'bind': '/root/.cache/huggingface/hub/models--openai-community--gpt2', 'mode': 'ro'},
            'models--openai-community--gpt2-large': {'bind': '/root/.cache/huggingface/hub/models--openai-community--gpt2-large', 'mode': 'ro'},
            }
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = huggingface_model_mounts(['openai-community/gpt2', 'openai-community/gpt2-large', 'openai-community/gpt2', 'openai-community/gpt2-large'])
        actual = {str(k).split('/')[-1]: v for k, v in actual.items()}
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)
    
    def test_ln_mount_for_none(self):
        expected = ''
        actual = _ln_huggingface_model_mounts(None)
        self.assertEqual(actual, expected)

    def test_ln_mount_for_empty_string(self):
        expected = ''
        actual = _ln_huggingface_model_mounts('')
        self.assertEqual(actual, expected)

    def test_ln_mount_for_empty_string_after_strip(self):
        expected = ''
        actual = _ln_huggingface_model_mounts('    ')
        self.assertEqual(actual, expected)

    def test_ln_mount_for_non_existing_model_throwns_exception(self):
        with self.assertRaises(Exception) as context:
            _ln_huggingface_model_mounts('this-model-does-not-exist')
        self.assertTrue('this-model-does-not-exist' in str(context.exception))

    def test_ln_mounts_for_single_existing_model(self):
        expected = 'mkdir -p /root/.cache/huggingface/hub/; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s <NORMALIZED>/hub/models--openai-community--gpt2 /root/.cache/huggingface/hub/models--openai-community--gpt2; echo "mounted 1 models"'
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = _ln_huggingface_model_mounts('openai-community/gpt2')
        actual = actual.replace(str(Path('tests/resources/hf').absolute()), '<NORMALIZED>')
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_single_existing_model_redundant(self):
        expected = 'mkdir -p /root/.cache/huggingface/hub/; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s <NORMALIZED>/hub/models--openai-community--gpt2 /root/.cache/huggingface/hub/models--openai-community--gpt2; echo "mounted 1 models"'
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = _ln_huggingface_model_mounts('openai-community/gpt2 openai-community/gpt2   openai-community/gpt2')
        actual = actual.replace(str(Path('tests/resources/hf').absolute()), '<NORMALIZED>')
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_multiple_existing_models(self):
        expected = 'mkdir -p /root/.cache/huggingface/hub/; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s <NORMALIZED>/hub/models--openai-community--gpt2 /root/.cache/huggingface/hub/models--openai-community--gpt2; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2-large; ln -s <NORMALIZED>/hub/models--openai-community--gpt2-large /root/.cache/huggingface/hub/models--openai-community--gpt2-large; echo "mounted 2 models"'
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = _ln_huggingface_model_mounts('openai-community/gpt2 openai-community/gpt2-large')
        actual = actual.replace(str(Path('tests/resources/hf').absolute()), '<NORMALIZED>')
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_multiple_existing_models_redundant(self):
        expected = 'mkdir -p /root/.cache/huggingface/hub/; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s <NORMALIZED>/hub/models--openai-community--gpt2 /root/.cache/huggingface/hub/models--openai-community--gpt2; rm -Rf /root/.cache/huggingface/hub/models--openai-community--gpt2-large; ln -s <NORMALIZED>/hub/models--openai-community--gpt2-large /root/.cache/huggingface/hub/models--openai-community--gpt2-large; echo "mounted 2 models"'
        os.environ['HF_HUB_CACHE'] = 'tests/resources/hf/hub'
        actual = _ln_huggingface_model_mounts('openai-community/gpt2 openai-community/gpt2-large openai-community/gpt2 openai-community/gpt2-large openai-community/gpt2 openai-community/gpt2-large')
        actual = actual.replace(str(Path('tests/resources/hf').absolute()), '<NORMALIZED>')
        del os.environ['HF_HUB_CACHE']
        self.assertEqual(actual, expected)



def print_license_agreement(url):
    if not url:
        return
    if url and 'longeval' in url or 'leipzig-topics-small-20240119' in url or 'jena-topics-20231026-test' in url or 'jena-topics-small-20240119-training' in url or 'leipzig-topics-20231025-test' in url or 'training-20231104-training' in url or 'validation-20231104-training' in url:
        print('The download is derived from The LongEval Dataset under the "Qwant LongEval Attribution-NonCommercial-ShareAlike License". Hence, the download is also under this License. By using it, you agree to the terms of this license. Please find details at: https://lindat.mff.cuni.cz/repository/xmlui/page/Qwant_LongEval_BY-NC-SA_License')


from tira.rest_api_client import Client
from tira.tira_redirects import redirects
import unittest
from contextlib import redirect_stdout
import io

def capture_potential_license_agreement(url):
    f = io.StringIO()
    with redirect_stdout(f):
        redirects(url=url)
    
    return f.getvalue()

class PrintLicenseAgreementTest(unittest.TestCase):
    def test_no_license_agreement_for_unrelated_url(self):
        url = 'https://webis.de/'
        expected = ''

        actual = capture_potential_license_agreement(url)
        self.assertEqual(expected, actual)

    def test_no_license_agreement_for_run_download_with_non_license_dataset(self):
        url = 'https://www.tira.io/task/ir-benchmarks/user/fschlatt/dataset/antique-test-20230107-training/download/2024-01-15-15-48-32.zip'
        expected = ''

        actual = capture_potential_license_agreement(url)
        self.assertEqual(expected, actual)

    def test_qwant_license_agreement_for_run_download_01(self):
        expected = 'The download is derived from The LongEval Dataset under the "Qwant LongEval Attribution-NonCommercial-ShareAlike License". Hence, the download is also under this License. By using it, you agree to the terms of this license. Please find details at: https://lindat.mff.cuni.cz/repository/xmlui/page/Qwant_LongEval_BY-NC-SA_License\n'
        datasets = ['longeval-train-20230513-training', 'longeval-short-july-20230513-training', 'longeval-long-september-20230513-training', 'longeval-heldout-20230513-training', 'longeval-2023-08-20240418-training', 'longeval-2023-06-20240418-training', 'jena-topics-20231026-test', 'jena-topics-small-20240119-training', 'leipzig-topics-20231025-test', 'leipzig-topics-small-20240119-training', 'training-20231104-training', 'validation-20231104-training', 'longeval-tiny-train-20240315-training', 'longeval-2023-08-20240423-training', 'longeval-2023-06-20240423-training',]

        for dataset in datasets:
            url = f'https://www.tira.io/task/ir-benchmarks/user/fschlatt/dataset/{dataset}/download/2024-01-15-15-48-32.zip'
            actual = capture_potential_license_agreement(url)
            self.assertEqual(expected, actual, dataset)


